//---------------------------------------------------------------------------
#include <condefs.h>
#pragma hdrstop
#define Library

// Pour ajouter un fichier  la bibliothque utilisez l'option 'Ajouter au projet' dans le menu Projet.

#include <conio.h>
#include <iostream.h>
#include <fstream.h>
#include <math.h>

#include <string.h>

#include "strFile.h"
#include "Filter.hxx"
#include "Complex.hxx"
#include "Signal1D.hxx"
#include <stdio.h>

Signal1D::Signal1D() {
    dim=0;
    data=NULL;
    empty=true;
    drawable=false;
    buffer=NULL;
    bufWidth=0;
}

Signal1D::Signal1D(unsigned long d) {
    dim=d;
    data=new Complex[dim];
    for (unsigned long i=0;i<dim;i++)
        data[i]=Complex(0.0,0.0);
    empty=false;
    drawable=false;
    buffer=NULL;
    bufWidth=0;
}

Signal1D::~Signal1D() {
    if (!empty) {
        delete[] data;
        data=NULL;
    }
    if (bufWidth>0) {
        delete[] buffer;
        buffer=NULL;
    }
}

void Signal1D::tfr() {
    fourierTransform(1);
}

void Signal1D::tfrInv() {
    fourierTransform(-1);
}

Complex Signal1D::operator[](unsigned long i) const {
    return data[i];
}

void Signal1D::dif(const Signal1D &a,const Signal1D &b) {
    for (unsigned long i=0;i<dim;i++)
        data[i].re=a.data[i].re-b.data[i].re;
}

void Signal1D::operator=(const Signal1D &s) {

    if ((s.empty)&&(!empty)) {
        delete[] data;
        data=NULL;
        delete[] buffer;
        buffer=NULL;
        empty=true;
        drawable=false;
        dim=0;
        height=0;
        width=0;
        bufWidth=0;
    }
    else {
        if ((!s.empty)&&(empty))
            data=new Complex[s.dim];
        else if (dim!=s.dim) {
            delete[] data;
            data=NULL;
            data=new Complex[s.dim];
            }
        dim=s.dim;
        height=s.height;
        width=s.width;
        empty=false;
        drawable=s.drawable;

        for (unsigned long i=0;i<dim;i++)
            data[i]=s.data[i];

        delete[] buffer;
        buffer=NULL;
        bufWidth=s.bufWidth;
        if (s.bufWidth>0) {
            buffer=new char[bufWidth];
            for (unsigned int i=0;i<bufWidth;i++)
                buffer[i]=s.buffer[i];
        }
    }
}

unsigned long Signal1D::dimension() const {
    return dim;
}

float Signal1D::module(long i) const {
    return sqrt(data[i].re*data[i].re+data[i].im*data[i].im);
}

void Signal1D::load(const char fileName[]) {
	FILE *file;
	int w,h,offset;
    __int16 *tmpBuffer;

	if ((file=fopen(fileName,"rb"))==0) return; // Code Erreur Ouverture fichier

	// lecture des parametres
	delete[] data;
    data=NULL;

	fScanIntParam(file,"\\Data offset:",&offset);
	fScanIntParam(file,"\\Valid data len X:",&w);
	fScanIntParam(file,"\\Valid data len Y:",&h);

    // initialisation des parametres du signal
    width=(unsigned int)w;
    height=(unsigned int)h;
    dim=w*h;

    // initialisation de l'image
	data=new Complex[dim];

	// repositionnement du pointeur de fichier

	fseek(file,0,SEEK_SET);
    delete[] buffer;
    buffer=NULL;
    bufWidth=offset;

    if (bufWidth>0) {
        buffer=new char[bufWidth];
        fread(buffer,sizeof(char),bufWidth,file);
    }

	fseek(file,offset,SEEK_SET);

	// lecture des valeurs binaires
    tmpBuffer =new __int16[dim];
    fread(tmpBuffer,sizeof(__int16),dim,file);
    for (unsigned int i=0;i<dim;i++)
        data[i]=Complex((float)tmpBuffer[i],0.0);
    delete[] tmpBuffer;


    // fermeture du fichier
	fclose(file);
    empty=false;
    drawable=true;
	return; // Code erreur lecture OK
}

float Signal1D::getPix(unsigned int l,unsigned int c) const {
    if ((l>=height)||(c>=width))
        return 0.0;
    return data[l*width+c].re;
}

float Signal1D::getPixNormalized(unsigned int l,unsigned int c,unsigned int w,unsigned int h) {
    unsigned long x,y;

    if ((l>=height)||(c>=width))
        return 0.0;
    x=(width-1)*c/w;
    y=(height-1)*l/h;

    if (x>=width) x=width-1;
    if (y>=height) y=height-1;
    
    return data[y*width+x].re;
}

bool Signal1D::isDrawable() const {
    return drawable;
}

bool Signal1D::isEmpty() const {
    return empty;
}

float Signal1D::min() const {
    float m=data[0].re;

    for (unsigned long i=1;i<dim;i++)
        if (m>data[i].re) m=data[i].re;

    return m;
}

float Signal1D::max() const {
    float m=data[0].re;

    for (unsigned long i=0;i<dim;i++)
        if (m<data[i].re) m=data[i].re;

    return m;
}

//----------------------------------------------

void Signal1D::fourierTransform(char sens) {
    unsigned long i,tmp;
    unsigned int n=expTwoDim();
    unsigned int j;

    if (n==0) return;

    // Initialisation du tableau d'indices
    unsigned long *Map=new unsigned long[dim];

    for (i=0;i<dim;i++) {
        Map[i]=0;
        tmp=i;
        for (j=0;j<n;j++) {
            Map[i]=(Map[i]<<1)|(tmp&1);
            tmp=tmp>>1;
        }
    }

    // Calcul des W
    double s;
    Signal1D W(dim/2);

    if (sens==-1) s=-1.0;
        else s=1.0;
    for (i=0;i<dim/2;i++)
        W.data[i]=Complex((double)cos(s*i*2*M_PI/dim),
                          (double)sin(s*i*2*M_PI/dim));


    // Initialisation du spectre
    Complex *sTmp=new Complex[dim];
    for (i=0;i<dim;i++)
        sTmp[i]=data[i];
    for (i=0;i<dim;i++)
        data[i]=sTmp[Map[i]];
    delete[] sTmp;

    // Transforme
    unsigned long nombre_blocs,indice,deca,nb_el_ds_bloc,k;
    Complex p,m;

    for (i=0;i<n;i++) {
        nb_el_ds_bloc=1<<(i+1);

        for (j=0;j<dim;j+=nb_el_ds_bloc) {
            for (k=0;k<nb_el_ds_bloc/2;k++) {
                p=data[j+k]+data[j+k+nb_el_ds_bloc/2];
                m=data[j+k]-data[j+k+nb_el_ds_bloc/2];
                data[j+k]=p;
                data[j+k+nb_el_ds_bloc/2]=m;
            }
        }

        if (i<n-1) {
	        nombre_blocs=(unsigned long)pow(2,n-i-2);
            deca=(unsigned long)pow(2,i+1);

	        for (j=1;j<=nombre_blocs;j++) {
		        indice=(2*j-1)*deca;
		        for (k=0;k<dim/2;k+=nombre_blocs) {
                    data[indice]=data[indice]*W.data[k];
    			    indice++;
	    	    }
	        }
        }
    }

    if (sens<0) {
        for (i=0;i<dim;i++)
            data[i]=data[i]/(double)dim;
    }

    delete[] Map;
}

unsigned int Signal1D::expTwoDim() {
    unsigned long i=2;
    unsigned int n=1;

    while (i<=dim) {
        if (i==dim) return n;
        i=i<<1;
        n+=1;
    }
    return 0;
}

void Signal1D::filter(Filter f) {

    if (f.automatic) {
        double delta=sqrt(pow((max()-min())*f.tolerance,2));
        autoLoFilter(dim/height*2,delta);
        }
    else {
        if (f.lo)
            loFilter(f.width*width/128);
        else if (f.hi)
            hiFilter(f.width*width/128);
    }
}

void Signal1D::computeNoise(Filter f) {
    double delta=sqrt(pow((max()-min())*f.tolerance,2));
    double median=0.0;
    unsigned long i;

    for (i=dim/2-dim/8;i<dim/2+dim/8;i++)
        median+=data[i].re;
    median/=(dim/4);

    i=0;
    while ((pow(data[i].re,2)>pow(median,2)+delta)&&(i<width))
        i++;

    raz(i,dim-i);
}


void Signal1D::loFilter(int width) {
    raz(0,width/2);
    raz(dim-width/2-1,dim-1);
}

void Signal1D::autoLoFilter(int width,float delta) {
    double median=0.0;
    unsigned long i;

    for (i=dim/2-dim/8;i<dim/2+dim/8;i++)
        median+=data[i].re;
    median/=(dim/4);

    i=0;
    while ((pow(data[i].re,2)>pow(median,2)+delta)&&(i<width))
        i++;

    loFilter(i);
}

void Signal1D::hiFilter(int width) {
    raz(dim/2-width/2,dim/2+width/2);
}

void Signal1D::raz(unsigned long mini,unsigned long maxi) {
    for (unsigned long i=mini;i<=maxi;i++)
        data[i].raz();
}

void Signal1D::cutMax(float M) {
    for (unsigned long i=0;i<height;i++) {
        int k=0;
        float med=0.0;
        for (unsigned long j=0;j<width;j++)
            if (data[j+i*height].re<M) {
                med=med+data[j+i*height].re;
                k++;
            }
        if (k>0) med=med/k;
        else med=0;
        
        for (unsigned long j=0;j<width;j++)
            if (data[j+i*height].re>M) data[j+i*height].re=med;
    }
}

// operateur affichage dans flux
ostream& operator<<(ostream &os,Signal1D &s) {
    if (s.isEmpty()) return os;

    os << s[0];
    if (s.dimension()>1)
        for (unsigned long i=1;i<s.dimension();i++)
            os << " " << s[i];

    return os;
}

