//_____________________________________________________________________________
/**
 * Filename : Signal1D.cxx
 * Description : 
 * @author: Philippe COVAL (http://RzR.online.FR) & Frederic MORALES
 * Date: 2000.02.20
 **/
#include "Signal1D.hxx"

//-----------------------------------------------------------------------------
Signal1D::Signal1D() {
  _data = NULL;
  _size = 0;
  _iref_afm = NULL ;
  //_header = NULL;
  _iw = 0;
  _ih = 0;
}

Signal1D::~Signal1D() {
  if ( _data != NULL) delete[] _data;
  _data = NULL ;
  _iref_afm = NULL ;
}

//-----------------------------------------------------------------------------

Signal1D::Signal1D(unsigned long dim) {
  _size=dim;
  _data=new Complex[_size];
  _iref_afm = NULL ;
  _iw = 0;
  _ih = 0;

  //  _header = NULL;
}
/** Copy Constructor */

Signal1D::Signal1D(const Signal1D &sig) {
  _iref_afm = sig._iref_afm ;
  _iw = sig._iw;
  _ih = sig._ih;
  _size = sig._size;
  _data = new Complex[_size];
  for (int i=0;i<(int) _size;i++)
    _data[i] = Complex( sig._data[i]);
}


Signal1D::Signal1D(ImageGray &img) {
  int i;
  _iref_afm = NULL ;
  _iw = img.getWidth();
  _ih = img.getHeight();
  _iref_afm = NULL ;
  //  _header = NULL;
  
  _size= (unsigned long) _ih * _iw ;
  _data=new Complex[_size];

  for (int h=0 ; h < _ih ; h++)
    for (int w=0 ; w < _iw ; w++) {
      i = h *  _iw   + w;
      _data[i] = Complex( (double) img[h][w] , 0 );
    }
}
//-----------------------------------------------------------------------------
Signal1D::Signal1D(ImageAFM &img) {
  debugf("Signal1D::Signal1D(ImageAFM &img) \n");
  int i=0;
  _iref_afm = (ImageAFM*)  &img;

  _iw = img.getWidth();
  _ih = img.getHeight() ;


  //debugf("%s",img._header);
  _size= (unsigned long) _ih * _iw ;
  _data=new Complex[_size];

  for (int h=0 ; h < _ih ; h++)
    for (int w=0 ; w < _iw ; w++) {
      i = h *  _iw   + w;
      _data[i] = Complex( (double) img[h][w] , 0 );
      //	debugf(" complexe : %d %f",img[h][w],_data[i].re);
    }
  
  //debugf("Signal1D::Signal1D(ImageAFM &img) : %d %d = %d \n",iw,ih,_size);
}
//-----------------------------------------------------------------------------
ImageAFM* Signal1D::getImageAFM() {
  int i;
  typeAfm t;
  debugf("... ImageAFM* Signal1D::getImageAFM(%d,%d) \n",_iw,_ih);
  debugf(" %s ",_iref_afm );
  if (_iref_afm == NULL) return NULL ;

  debugf("... get the source header\n");
  ImageAFM *img = new ImageAFM(*_iref_afm);
  int ih = img->getHeight();
  int iw = img->getWidth();

  debugf("... fill the pixels %d %d\n",iw,ih); 
  for (int h=0 ; h < ih ; h++)
    for (int w=0 ; w < iw ; w++) {
      i = h *  iw  + w;

      t =     (typeAfm) _data[i].re;
      // debugf("<%d=%e>\n",i,t);
      (*img)[h][w]  = t;
    }
  //  debugf(" ok : %s\n",img->_header); 
  debugf("getImage: over\n");
  return (img) ;
}

//-----------------------------------------------------------------------------

ImageGray *Signal1D::getImageDyn(int iw, int ih) {
  //  debugf("ImageGray *Signal1D::getImage(int iw, int ih) \n");
  int i;
  unsigned char tuc;
  double coef;
  ImageGray *img = new ImageGray(iw,ih);
  
  double max=this->getMax();
  double min=this->getMin();

  coef = (double) 255./ (double) ( abs ( (int) (max -min)));

  debugf("Etalement dyn : %d %d %e\n",(int) min,(int) max,  coef );

  double comp;
 
  for (int h=0 ; h < ih ; h++)
    for (int w=0 ; w < iw ; w++) {
      
      i = h *  iw  + w;
      comp = _data[i].re;
      comp = (( comp - (double) min ) * (double) coef );
      if ( comp > 255 ) comp = 255;
      if ( comp < 0 ) comp = 0;
      tuc =  (unsigned char) comp ;

      (*img)[h][w]  = tuc;
    }
  debugf("getImage(%d, %d) : over\n",iw,ih);
  return (img) ;
}

//---------------------------------------------------------------------------

ImageGray *Signal1D::getImageThreshold( int iw, int ih) {
  //  debugf("ImageGray *Signal1D::getImage(int iw, int ih) \n");
  int i;
  unsigned char tuc;
  ImageGray *img = new ImageGray(iw,ih);
 
  double comp;
 
  for (int h=0 ; h < ih ; h++)
    for (int w=0 ; w < iw ; w++) {
      
      i = h *  iw  + w;
      comp = _data[i].re;
      if ( comp > 255 ) comp = 255;
      if ( comp < 0 ) comp = 0;
      tuc =  (unsigned char) comp ;

      (*img)[h][w]  = tuc;
    }
  debugf("getImage(%d, %d) : over\n",iw,ih);
  return (img) ;
}

//-----------------------------------------------------------------------------

double Signal1D::getMax() {
  double v,t;
  if ( _data==NULL) return 0;
  v =  _data[0].re;

  for (int i=1;i< (int) _size;i++) {
    t =  _data[i].re;
    if (v <  t ) v = t ;
  }
  // debugf("getmax: %d \n",(int) v);
  return v;
}

double Signal1D::getMin() {
  double  v,t;
  if ( _data==NULL) return 0;
  v =  _data[0].re;

  for (int i=1;i< (int) _size ;i++) {
    t =  _data[i].re;
    if (v >  t ) v = t ;
  }
  //  debugf("getmin: %d \n",(int) v);
  return v;
}


unsigned int Signal1D::getNoise(int width, float delta) {
  double median=0.0;
  unsigned int i;

  for (i=_size/2-_size/8;i<_size/2+_size/8;i++)
    median+=_data[i].re;
  median/=(_size/4);

  i=1;
  while ((pow(_data[i].re,2)
	  > pow(median,2)+ delta)
	 &&(i< (unsigned int) width))
    i++;
  return i;
}


//---------------------------------------------------------------------------
Signal1D* Signal1D::tfr() {
  return fourierTransform(1);
}

Signal1D* Signal1D::tfrInv() {
  return fourierTransform(-1);
}

void Signal1D::raz(unsigned long mini,unsigned long maxi) {
  for (unsigned long i=mini;i<=maxi;i++)
    _data[i].raz();
}

void Signal1D::cutHigh(int width) {
  int center = _size / 2;
  int offset = width / 2;
  raz(center - offset, center + offset);
}

void Signal1D::cutLow(int width) {
  int offset = width / 2;
  raz(0 , offset);
  raz(_size - offset, _size );
}


//-----------------------------------------------------------------------------

ostream& operator<<(ostream &os,Signal1D &sig) {
  for (unsigned long i=0;i<sig._size;i++)
    os << sig._data[i];
  return os;
}

istream& operator>>(istream &is,Signal1D &sig) {
  for (unsigned long i=0;i<sig._size;i++)
    is >> sig._data[i];
  return is;
}

//-----------------------------------------------------------------------------

Signal1D* Signal1D::fourierTransform(char sens) {
  unsigned long i,tmp;
  unsigned int n=expTwoDim();
  unsigned int j;

  if (n==0) return new Signal1D();

  //debugf("Initialisation du tableau d'indices \n");
  unsigned long *Map=new unsigned long[_size];

  for (i=0;i<_size;i++) {
    Map[i]=0;
    tmp=i;
    for (j=0;j<n;j++) {
      Map[i]=(Map[i]<<1)|(tmp&1);
      tmp=tmp>>1;
    }
  }

  // Calcul des W
  float s;
  Signal1D W(_size/2);

  if (sens==-1) s=-1.0;
  else s=1.0;
  for (i=0;i<_size/2;i++)
    W._data[i]=Complex((float)cos(s*i*2*M_PI/_size),
		       (float)sin(s*i*2*M_PI/_size));


  // Initialisation du spectre
  Signal1D *spectre=new Signal1D(_size);
  for (i=0;i<_size;i++)
    spectre->_data[i]=_data[Map[i]];

  // Transforme
  unsigned long nombre_blocs,indice,deca,nb_el_ds_bloc,k;
  Complex p,m;

  for (i=0;i<n;i++) {
    nb_el_ds_bloc=1<<(i+1);

    for (j=0;j<_size;j+=nb_el_ds_bloc) {
      for (k=0;k<nb_el_ds_bloc/2;k++) {
	p=spectre->_data[j+k]+spectre->_data[j+k+nb_el_ds_bloc/2];
	m=spectre->_data[j+k]-spectre->_data[j+k+nb_el_ds_bloc/2];
	spectre->_data[j+k]=p;
	spectre->_data[j+k+nb_el_ds_bloc/2]=m;
      }
    }

    if (i<n-1) {
      nombre_blocs=(unsigned long)pow(2,n-i-2);
      deca=(unsigned long)pow(2,i+1);

      for (j=1;j<=nombre_blocs;j++) {
	indice=(2*j-1)*deca;
	for (k=0;k<_size/2;k+=nombre_blocs) {
	  spectre->_data[indice]=spectre->_data[indice]*W._data[k];
	  indice++;
	}
      }
    }
  }

  if (sens<0) {
    for (i=0;i<_size;i++)
      spectre->_data[i]=spectre->_data[i]/(float)_size;
  }

  delete[] Map; Map = NULL ;
  //debugf("... mods rzr\n");
  for (i=0;i<_size;i++)
    _data[i] = spectre->_data[i];
  delete spectre ; spectre=NULL;
  //debugf("return");
  return this;
}

//-----------------------------------------------------------------------------

unsigned int Signal1D::expTwoDim() {
  unsigned long i=2;
  unsigned int n=1;

  while (i<=_size) {
    if (i==_size) return n;
    i=i<<1;
    n+=1;
  }
  return 0;
}
//
/* ???

   int Signal1D::conversionInverse(ImageAFM *imgAFM) {
   if ((imgAFM->getWidth())*(imgAFM->getHeight())!=(int)size) 
   return 1;
   for (int i=0;i<imgAFM->getHeight();i++)
   for (int j=0;j<imgAFM->getWidth();j++)
   imgAFM->set(i,j,sqrt(
   pow((_data[i*imgAFM->getWidth()+j].re),2)
   +pow((_data[i*imgAFM->getWidth()+j].im),2)));
   return 0;
   }

   //
*/

//_____________________________________________________________________________
