//_____________________________________________________________________________
/**  <pre>
 * Filename: Signal1D.hxx
 * Description : Complex signal in 1 dimenssion
 * @author: Philippe COVAL (http://RzR.online.FR) & Frederic MORALES
 * Date: 2000.02.20
 *  </pre>
 **/
#ifndef __SIGNAL1D__
#define __SIGNAL1D__

#include "Complex.hxx"
#include "ImageGeneric.hxx"
#include "ImageGray.hxx"
#include "ImageAFM.hxx"


#include <iostream.h>
#include <math.h>
#include <string.h>

#include "compiler.h"

//-----------------------------------------------------------------------------
class Signal1D {
  
  /// members
public:
  ImageAFM  *_iref_afm;
private:
  unsigned long _size;
  Complex *_data;
  int _iw;
  int _ih;
  
  //ImageGeneric  *_iref_afm; /Signal1D.hxx:29: syntax error before `*'/ 

public:
  ///constructor :
  Signal1D();
  Signal1D(unsigned long dim);
  Signal1D(ImageGray &img);
  Signal1D(ImageAFM &img);
  Signal1D(const Signal1D &sig);
  ~Signal1D();

  /// methods
  Signal1D* tfr();
  Signal1D* tfrInv();
  void cutLow(int width);
  void cutHigh(int width);

  ImageGray* getImageDyn(int iw, int ih);
  ImageGray* getImageThreshold(int iw, int ih);
  ImageAFM* getImageAFM();

  unsigned long getSize() { return _size; }
  double getMax();
  double getMin();
  unsigned int getNoise(int width, float delta) ;

  friend ostream& operator<<(ostream &os,Signal1D &sig);
  friend istream& operator>>(istream &is,Signal1D &sig);

  void raz(unsigned long mini, unsigned long maxi);

protected:
  Signal1D *fourierTransform(char sens);
  unsigned int expTwoDim();
};
#endif
//_____________________________________________________________________________
