//_____________________________________________________________________________
/** <pre>
 * Filename: gui_callbacks.hxx
 * Description:  The callback functions are separated here
 * just because it is easier to develop into a real editor than fluid
 * @author: Philippe COVAL (http://RzR.online.FR) & Frederic MORALES
 * </pre>
 **/

#include "gui.hxx"

#include <FL/fl_ask.H>
#include <FL/fl_file_chooser.H>

#include "ImageAFM.hxx"
#include "ImageGray.hxx"
#include "Signal1D.hxx"

#define debugf printf
//-----------------------------------------------------------------------------

void gui::cbQuit(Fl_Widget* W,void* Obj) { 
  exit(0);
}

/**
 * The file format from the AFM software
 **/

void gui::cbLoadAfm (Fl_Widget* W,void* Obj) {
  char *f;
  debugf("void gui::cbLoadAfm (Fl_Widget* W,void* Obj) \n");

  f = fl_file_chooser("Pick a file","*","../data/img_afm/");
  if (!f) return;

  ((gui *)Obj)->s->cleanAFM();
  ((gui *)Obj)->s->clean();
  ((gui *)Obj)->t->clean();



  debugf("... loading new image\n");
  ImageAFM *img = new ImageAFM(f);

  int w,h;
  w = img->getWidth() ;
  h = img->getHeight() ;


  debugf("... infos \n");
  char s[256];
 
  sprintf ( s, "Image: [ %d x %d ] (%s) ",w,h,f);
 
  ((gui *)Obj)->texto->value(s);

  debugf("... alloc dest \n");

  ((gui *)Obj)->s->_iw = w;
  ((gui *)Obj)->s->_ih = h;
  ((gui *)Obj)->t->_iw = w;
  ((gui *)Obj)->t->_ih = h;

  debugf("... signaling \n");

  ((gui *)Obj)->s->setSignal(*img);

  debugf("... showing \n");
  ((gui *)Obj)->s->dispDyn();
}
//-----------------------------------------------------------------------------

void gui::cbSaveAfm (Fl_Widget* W,void* Obj) {
  char *f;
  debugf("void gui::cbSaveAfm (Fl_Widget* W,void* Obj) \n");

  Signal1D *sig = (((gui *)Obj)->t->_sig);
  if ( sig == NULL ) { fl_alert("Process the image 1st"); return; }

  f = (char *) fl_file_chooser("Pick a file","*","../data/img_afm/");
  if (!f) return;
  
  int w,h;
  w = ((gui *)Obj)->t->_iw;
  h = ((gui *)Obj)->t->_ih;
  
  debugf("... extract the image of the signal \n");
  ImageAFM *i = sig->getImageAFM();
  //debugf("... write header %s + data \n",i->_header );
  debugf("... write header + data \n");
  i->save(f);
  //delete i; i = NULL; // keep it for any other use
}


//-----------------------------------------------------------------------------

void gui::cbLoadPnm (Fl_Widget* W,void* Obj) {
  char *f;
  
  debugf("void gui::cbLoadPnm (Fl_Widget* W,void* Obj) \n");
  f = fl_file_chooser("Pick a file","*.pnm","../data/img_pnm/");
  if (!f) return;

	((gui *)Obj)->s->cleanAFM();
  ((gui *)Obj)->s->clean();
  ((gui *)Obj)->t->clean();

  debugf("... loading new image\n");

  ImageGray *img = new ImageGray(f);
  int w,h;
  w = img->getWidth() ;
  h = img->getHeight() ;


  //debugf("... infos \n");
  char s[256];
   sprintf ( s, "Image: [ %d x %d ] (%s) ",w,h,f);

  ((gui *)Obj)->texto->value(s);

  debugf("... alloc dest \n");
  ((gui *)Obj)->s->_iw = w;
  ((gui *)Obj)->s->_ih = h;
  ((gui *)Obj)->t->_iw = w;
  ((gui *)Obj)->t->_ih = h;

  debugf("... showing \n");
  ((gui *)Obj)->s->setSignal(*img);
  ((gui *)Obj)->t->_sig = new Signal1D(  *((gui *)Obj)->s->_sig );
  ((gui *)Obj)->s->dispThreshold();
}
//-----------------------------------------------------------------------------

/**
 * find the noise in the frequency transform
 **/


void gui::cbAuto(Fl_Widget* W,void* Obj) {
  if (((gui *)Obj)->s->_img == NULL ) {
    fl_alert("1/ load a file ");
    return ; 
  }
  
  ((gui *)Obj)->t->clean();

  Signal1D *sig = new Signal1D ( *  ((gui *)Obj)->s->_sig);
  //debugf(" %s ",sig->_iref_afm->_header);

  debugf("... fft  \n" );
  sig->tfr();

  char str[256];
  const char *strtmp;
  strtmp = ((gui *)Obj)->texto->value(); 

  double delta =((gui *)Obj)->filter_width->value() / 100 ; // 1/8 is ok

  int ls = sig->getNoise(((gui *)Obj)->s->_iw , (float) delta);
  int hs = (int) sig->getSize() - ls;
 
  ((gui *)Obj)->filter_width->value((int)hs);
  sprintf ( str, "%s\nDetected noise bandwidth : LO=%d HI=%d\n",strtmp,ls,hs);
  ((gui *)Obj)->texto->value(str);
  ((gui *)Obj)->texto->show();
  ((gui *)Obj)->texto->redraw();

  //  debugf("... filtering %d\n",s);



  debugf("... filtering \n");
  sig->cutLow((int) ls);

  debugf("... fftr\n");
  sig->tfrInv();

  debugf("... display\n");
  ((gui *)Obj)->t->_sig = sig ;
  ((gui *)Obj)->t->dispDyn();
}


//-----------------------------------------------------------------------------


void gui::cbAutoLo(Fl_Widget* W,void* Obj) {

  if (((gui *)Obj)->s->_img == NULL ) {
    fl_alert("1/ load a file ");
    return ; 
  }
  ((gui *)Obj)->t->clean();
  double s =((gui *)Obj)->filter_width->value();
  
  Signal1D *sig = new Signal1D ( * ( ((gui *)Obj)->s->_sig) );

  //debugf("... signal processing \n");
  debugf("... fft  \n" );
  sig->tfr();
  debugf("... filtering %f \n",s);
  sig->cutLow((int) s );
  debugf("... fftr\n");
  sig->tfrInv();

  debugf("... display\n");
  ((gui *)Obj)->t->_sig = sig ;
  ((gui *)Obj)->t->dispDyn();
}

//-----------------------------------------------------------------------------
void gui::cbAutoHi(Fl_Widget* W,void* Obj) {
  if (((gui *)Obj)->s->_img == NULL ) {
    fl_alert("1/ load a file ");
    return ; 
  }

  ((gui *)Obj)->t->clean();
  double s =((gui *)Obj)->filter_width->value();
  
  Signal1D *sig = new Signal1D ( * ( ((gui *)Obj)->s->_sig) );

  //debugf("... signal processing \n");
  debugf("... fft  \n" );
  sig->tfr();
  debugf("... filtering %f \n",s);
  sig->cutHigh((int) s );
  debugf("... fftr\n");
  sig->tfrInv();

  debugf("... display\n");
  ((gui *)Obj)->t->_sig = sig ;
  ((gui *)Obj)->t->dispDyn();
}

//-----------------------------------------------------------------------------



void gui::cbSavePnm(Fl_Widget* W,void* Obj) {
  char *f;
  if (((gui *)Obj)->t->_sig == NULL ) {
    fl_alert("1/ load a file \n2/ filter it");
    return ; 
  }

  f = fl_file_chooser("Pick a file","*.pnm","../data/img_pnm/");
  if (f) {
    int w,h;
    w = ((gui *)Obj)->t->_iw;
    h = ((gui *)Obj)->t->_ih;

    ImageGray *im = ((gui *)Obj)->t->_sig->getImageThreshold(w,h) ;
    im->savePNM(f);
  }
}

//-----------------------------------------------------------------------------


void gui::cbFFT(Fl_Widget* W,void* Obj) {
  
  if (((gui *)Obj)->s->_img == NULL ) {
    fl_alert("... Load a file 1st !");
    return ; 
  }

  //((gui *)Obj)->t->clean();
  debugf("... fft  \n" );
  Signal1D *sig = ((gui *)Obj)->t->_sig ;
  sig->tfr();
  ((gui *)Obj)->t->_sig = sig;
 
  debugf("... showing \n");
  ((gui *)Obj)->t->dispThreshold();
  debugf("... end : callback ftt \n");
}

//-----------------------------------------------------------------------------


void gui::cbFFTR(Fl_Widget* W,void* Obj) {
  
  if (((gui *)Obj)->t->_sig == NULL ) {
    fl_alert("1/ load file \n2/ tfr");
    return ; 
  }

  debugf("... fftR  \n" );
  ((gui *)Obj)->t->_sig->tfrInv();

  debugf("... showing \n");
  ((gui *)Obj)->t->dispDyn();

  debugf("... end : callback ftt \n");
}

//-----------------------------------------------------------------------------


void gui::cbCutLow(Fl_Widget* W,void* Obj) {
  if (((gui *)Obj)->t->_sig == NULL ) {
    fl_alert("1/ load a file \n2/ tranform it with 'FFT'");
    return ; 
  }


  double s =((gui *)Obj)->filter_width->value();
  debugf("... filtering %f\n",s);
  ((gui *)Obj)->t->_sig->cutLow((int) s);

  debugf("... showing \n");
  ((gui *)Obj)->t->dispThreshold();
  debugf("cbFilter : ok \n");

}

void gui::cbCutHigh(Fl_Widget* W,void* Obj) {
  if (((gui *)Obj)->t->_sig == NULL ) {
    fl_alert("1/ load a file \n2/ tranform it with 'FFT'");
    return ; 
  }

  double s =((gui *)Obj)->filter_width->value();
  debugf("... filtering %f\n",s);
  ((gui *)Obj)->t->_sig->cutHigh((int) s);

  debugf("... showing \n");
  ((gui *)Obj)->t->dispThreshold();
  debugf("cbFilter : ok \n");
}

//-----------------------------------------------------------------------------
